@@ -10,7 +10,7 @@ from .checker import ConnectionChecker
 
 from contextlib import contextmanager
 import random
-import select
+from select import select
 import socket
 import threading
 
@@ -19,15 +19,18 @@ class Client(object):
     '''A client for talking to NSQ over a connection'''
     def __init__(self,
         lookupd_http_addresses=None, nsqd_tcp_addresses=None, topic=None,
-        timeout=0.1, reconnection_backoff=None, **identify):
+        timeout=0.1, reconnection_backoff=None, auth_secret=None, **identify):
         # If lookupd_http_addresses are provided, so must a topic be.
         if lookupd_http_addresses:
             assert topic
 
         # Create clients for each of lookupd instances
         lookupd_http_addresses = lookupd_http_addresses or []
+        params = {}
+        if auth_secret:
+            params['access_token'] = auth_secret
         self._lookupd = [
-            nsqlookupd.Client(host) for host in lookupd_http_addresses]
+            nsqlookupd.Client(host, **params) for host in lookupd_http_addresses]
         self._topic = topic
 
         # The select timeout
@@ -37,6 +40,7 @@ class Client(object):
 
         # The options to send along with identify when establishing connections
         self._identify_options = identify
+        self._auth_secret = auth_secret
         # A mapping of (host, port) to our nsqd connection objects
         self._connections = {}
 
@@ -53,6 +57,7 @@ class Client(object):
             try:
                 # Find all the current producers on this instance
                 for producer in lookupd.lookup(topic)['producers']:
+                    logger.info('Found producer %s on %s', producer, lookupd)
                     producers.append(
                         (producer['broadcast_address'], producer['tcp_port']))
             except ClientException:
@@ -114,6 +119,7 @@ class Client(object):
         '''Connect to the provided host, port'''
         conn = connection.Connection(host, port,
             reconnection_backoff=self._reconnection_backoff,
+            auth_secret=self._auth_secret,
             **self._identify_options)
         if conn.alive():
             conn.setblocking(0)
@@ -172,7 +178,7 @@ class Client(object):
         # Not all connections need to be written to, so we'll only concern
         # ourselves with those that require writes
         writes = [c for c in connections if c.pending()]
-        readable, writable, exceptable = select.select(
+        readable, writable, exceptable = select(
             connections, writes, connections, self._timeout)
 
         # If we returned because the timeout interval passed, log it and return
