@@ -1,5 +1,4 @@
 import mock
-import unittest
 
 from nsq import client
 from nsq import response
@@ -7,25 +6,27 @@ from nsq import constants
 from nsq import exceptions
 from nsq.http import ClientException
 
-from common import FakeServerTest
+from common import HttpClientIntegrationTest, MockedConnectionTest
 from contextlib import contextmanager
 import socket
 
 
-class TestClientNsqd(FakeServerTest):
+class TestClientNsqd(HttpClientIntegrationTest):
     '''Test our client class'''
-    def connect(self):
+    nsqd_ports = (14150,)
+
+    def setUp(self):
         '''Return a new client'''
-        hosts = ['localhost:%s' % port for port in self.ports]
-        return client.Client(nsqd_tcp_addresses=hosts)
+        HttpClientIntegrationTest.setUp(self)
+        hosts = ['localhost:%s' % port for port in self.nsqd_ports]
+        self.client = client.Client(nsqd_tcp_addresses=hosts)
 
     def test_connect_nsqd(self):
         '''Can successfully establish connections'''
-        with self.identify():
-            connections = self.client.connections()
-            self.assertEqual(len(connections), 1)
-            for connection in connections:
-                self.assertTrue(connection.alive())
+        connections = self.client.connections()
+        self.assertEqual(len(connections), 1)
+        for connection in connections:
+            self.assertTrue(connection.alive())
 
     def test_add_added(self):
         '''Connect invokes self.connected'''
@@ -36,16 +37,14 @@ class TestClientNsqd(FakeServerTest):
 
     def test_add_existing(self):
         '''Adding an existing connection returns None'''
-        with self.identify():
-            connection = self.client.connections()[0]
-            self.assertEqual(self.client.add(connection), None)
+        connection = self.client.connections()[0]
+        self.assertEqual(self.client.add(connection), None)
 
     def test_remove_exception(self):
         '''If closing a connection raises an exception, remove still works'''
-        with self.identify():
-            connection = self.client.connections()[0]
-            with mock.patch.object(connection, 'close', side_effect=Exception):
-                self.assertEqual(self.client.remove(connection), connection)
+        connection = self.client.connections()[0]
+        with mock.patch.object(connection, 'close', side_effect=Exception):
+            self.assertEqual(self.client.remove(connection), connection)
 
     def test_honors_identify_options(self):
         '''Sends along identify options to each connection as it's created'''
@@ -54,7 +53,7 @@ class TestClientNsqd(FakeServerTest):
                 self.client, '_identify_options', {'foo': 'bar'}):
                 self.client.connect('foo', 'bar')
                 MockConnection.assert_called_with('foo', 'bar',
-                    reconnection_backoff=None, foo='bar')
+                    reconnection_backoff=None, auth_secret=None, foo='bar')
 
     def test_conection_checker(self):
         '''Spawns and starts a connection checker'''
@@ -63,27 +62,19 @@ class TestClientNsqd(FakeServerTest):
         self.assertFalse(checker.is_alive())
 
 
-class TestClientLookupd(FakeServerTest):
+class TestClientLookupd(HttpClientIntegrationTest):
     '''Test our client class'''
-    def connect(self):
+    def setUp(self):
         '''Return a new client'''
-        with mock.patch('nsq.client.nsqlookupd.Client') as MockClass:
-            MockClass.return_value.lookup.return_value = {
-                'producers': [{
-                    'broadcast_address': 'localhost',
-                    'tcp_port': self.ports[0]
-                }]
-            }
-            return client.Client(topic='foo',
-                lookupd_http_addresses=['http://localhost:1234'])
+        HttpClientIntegrationTest.setUp(self)
+        self.client = client.Client(topic=self.topic, lookupd_http_addresses=['http://localhost:14161'])
 
     def test_connected(self):
         '''Can successfully establish connections'''
-        with self.identify():
-            connections = self.client.connections()
-            self.assertEqual(len(connections), 1)
-            for connection in connections:
-                self.assertTrue(connection.alive())
+        connections = self.client.connections()
+        self.assertEqual(len(connections), 1)
+        for connection in connections:
+            self.assertTrue(connection.alive())
 
     def test_asserts_topic(self):
         '''If nsqlookupd servers are provided, asserts a topic'''
@@ -102,7 +93,7 @@ class TestClientLookupd(FakeServerTest):
     def test_discover_connected(self):
         '''Doesn't freak out when rediscovering established connections'''
         before = self.client.connections()
-        self.client.discover('foo')
+        self.client.discover(self.topic)
         self.assertEqual(self.client.connections(), before)
 
     def test_discover_closed(self):
@@ -111,25 +102,46 @@ class TestClientLookupd(FakeServerTest):
             conn.close()
         state = [conn.alive() for conn in self.client.connections()]
         self.assertEqual(state, [False])
-        self.client.discover('foo')
+        self.client.discover(self.topic)
         state = [conn.alive() for conn in self.client.connections()]
         self.assertEqual(state, [True])
 
+    def test_auth_secret(self):
+        '''If an auth secret is provided, it passes it to nsqlookupd'''
+        with mock.patch('nsq.client.nsqlookupd.Client') as MockClient:
+            client.Client(topic=self.topic, lookupd_http_addresses=['foo'], auth_secret='hello')
+            MockClient.assert_called_with('foo', access_token='hello')
 
-class TestClientMultiple(FakeServerTest):
+
+class TestClientMultiple(MockedConnectionTest):
     '''Tests for our client class'''
-    ports = (12345, 12346)
+    @contextmanager
+    def readable(self, connections):
+        '''With all the connections readable'''
+        value = (connections, [], [])
+        with mock.patch.object(client, 'select', return_value=value):
+            yield
 
-    def connect(self):
-        '''Return a new client'''
-        hosts = ['localhost:%s' % port for port in self.ports]
-        return client.Client(nsqd_tcp_addresses=hosts)
+    @contextmanager
+    def writable(self, connections):
+        '''With all the connections writable'''
+        value = ([], connections, [])
+        with mock.patch.object(client, 'select', return_value=value):
+            yield
+
+    @contextmanager
+    def exceptable(self, connections):
+        '''With all the connections exceptable'''
+        value = ([], [], connections)
+        with mock.patch.object(client, 'select', return_value=value):
+            yield
 
     def test_multi_read(self):
         '''Can read from multiple sockets'''
-        with self.identify():
-            self.servers[0].response('hello')
-            self.servers[1].response('hello')
+        # With all the connections read-ready
+        for conn in self.connections:
+            conn.response('hello')
+        with self.readable(self.connections):
             found = self.client.read()
             self.assertEqual(len(found), 2)
             for res in found:
@@ -138,34 +150,31 @@ class TestClientMultiple(FakeServerTest):
 
     def test_heartbeat(self):
         '''Sends a nop on connections that have received a heartbeat'''
-        with self.identify():
-            self.servers[0].response(constants.HEARTBEAT)
-            # Get the heartbeat and automatically send the NOP
+        for conn in self.connections:
+            conn.response(constants.HEARTBEAT)
+        with self.readable(self.connections):
             self.assertEqual(self.client.read(), [])
-            self.assertEqual(self.client.read(), [])
-            self.assertEqual(
-                self.servers[0].read(100), constants.NOP + constants.NL)
+            for conn in self.connections:
+                conn.nop.assert_called_with()
 
     def test_closes_on_fatal(self):
         '''All but a few errors are considered fatal'''
-        with self.identify():
-            self.servers[0].error(exceptions.InvalidException)
+        self.connections[0].error(exceptions.InvalidException)
+        with self.readable(self.connections):
             self.client.read()
-            states = set(conn.alive() for conn in self.client.connections())
-            self.assertEqual(states, set((True, False)))
+            self.assertFalse(self.connections[0].alive())
 
     def test_nonfatal(self):
         '''Nonfatal errors keep the connection open'''
-        with self.identify():
-            self.servers[0].error(exceptions.FinFailedException)
+        self.connections[0].error(exceptions.FinFailedException)
+        with self.readable(self.connections):
             self.client.read()
-            states = set(conn.alive() for conn in self.client.connections())
-            self.assertEqual(states, set((True,)))
+            self.assertTrue(self.connections[0].alive())
 
     def test_passes_errors(self):
         '''The client's read method should now swallow Error responses'''
-        with self.identify():
-            self.servers[0].error(exceptions.InvalidException)
+        self.connections[0].error(exceptions.InvalidException)
+        with self.readable(self.connections):
             res = self.client.read()
             self.assertEqual(len(res), 1)
             self.assertIsInstance(res[0], response.Error)
@@ -174,71 +183,58 @@ class TestClientMultiple(FakeServerTest):
     def test_closes_on_exception(self):
         '''If a connection gets an exception, it closes it'''
         # Pick a connection to have throw an exception
-        with self.identify():
-            connection = self.client.connections()[0]
-            with mock.patch.object(
-                connection, 'read', side_effect=exceptions.NSQException):
-                self.servers[0].response('hello')
-                self.servers[1].response('hello')
+        conn = self.connections[0]
+        with mock.patch.object(
+            conn, 'read', side_effect=exceptions.NSQException):
+            with self.readable(self.connections):
                 self.client.read()
-                self.assertFalse(connection.alive())
+                self.assertFalse(conn.alive())
 
     def test_closes_on_read_socket_error(self):
         '''If a connection gets a socket error, it closes it'''
         # Pick a connection to have throw an exception
-        with self.identify():
-            connection = self.client.connections()[0]
-            with mock.patch.object(
-                connection, 'read', side_effect=socket.error):
-                self.servers[0].response('hello')
-                self.servers[1].response('hello')
+        conn = self.connections[0]
+        with mock.patch.object(
+            conn, 'read', side_effect=socket.error):
+            with self.readable(self.connections):
                 self.client.read()
-                self.assertFalse(connection.alive())
+                self.assertFalse(conn.alive())
 
     def test_closes_on_flush_socket_error(self):
         '''If a connection fails to flush, it gets closed'''
         # Pick a connection to have throw an exception
-        with self.identify():
-            connection = self.client.connections()[0]
-            with mock.patch.object(
-                connection, 'flush', side_effect=socket.error):
-                with mock.patch.object(
-                    connection, 'pending', return_value=True):
-                    self.client.read()
-                    self.assertFalse(connection.alive())
+        conn = self.connections[0]
+        with mock.patch.object(
+            conn, 'flush', side_effect=socket.error):
+            with self.writable(self.connections):
+                self.client.read()
+                self.assertFalse(conn.alive())
 
     def test_read_writable(self):
         '''Read flushes any writable connections'''
-        with self.identify():
-            with mock.patch('nsq.client.select') as MockSelect:
-                connection = mock.Mock()
-                MockSelect.select.return_value = ([], [connection], [])
-                self.client.read()
-                connection.flush.assert_called_with()
+        with self.writable(self.connections):
+            self.client.read()
+            for conn in self.connections:
+                conn.flush.assert_called_with()
 
     def test_read_exceptions(self):
-        '''Read flushes any writable connections'''
-        with self.identify():
-            with mock.patch('nsq.client.select') as MockSelect:
-                connection = mock.Mock()
-                MockSelect.select.return_value = ([], [], [connection])
-                self.client.read()
-                connection.close.assert_called_with()
+        '''Closes connections with socket errors'''
+        with self.exceptable(self.connections):
+            self.client.read()
+            for conn in self.connections:
+                self.assertFalse(conn.alive())
 
     def test_read_timeout(self):
         '''Logs a message when our read loop finds nothing because of timeout'''
-        with self.identify():
-            with mock.patch('nsq.client.select') as MockSelect:
-                with mock.patch('nsq.client.logger') as MockLogger:
-                    MockSelect.select.return_value = ([], [], [])
-                    self.client.read()
-                    MockLogger.debug.assert_called_with('Timed out...')
+        with self.readable([]):
+            with mock.patch('nsq.client.logger') as MockLogger:
+                self.client.read()
+                MockLogger.debug.assert_called_with('Timed out...')
 
     def test_read_with_no_connections(self):
         '''Attempting to read with no connections'''
-        with self.identify():
-            with mock.patch.object(self.client, 'connections', return_value=[]):
-                self.assertEqual(self.client.read(), [])
+        with mock.patch.object(self.client, 'connections', return_value=[]):
+            self.assertEqual(self.client.read(), [])
 
     def test_random_connection(self):
         '''Yields a random client'''
@@ -284,54 +280,37 @@ class TestClientMultiple(FakeServerTest):
                     self.client.mpub('foo', *messages), ['response'])
                 connection.mpub.assert_called_with('foo', *messages)
 
-
-class TestClientNsqdReconnection(FakeServerTest):
-    '''Test our client class'''
-    def connect(self):
-        '''Return a new client'''
-        hosts = ['localhost:%s' % port for port in self.ports]
-        return client.Client(nsqd_tcp_addresses=hosts)
-
-    @contextmanager
-    def mocked_dead_connection(self):
-        '''Yields the sole mocked connection of the client'''
-        conn = mock.Mock()
-        with mock.patch.object(self.client, '_connections') as mock_connections:
-            mock_connections.get.return_value = conn
-            conn.alive.return_value = False
-            yield conn
-
     def test_not_ready_to_reconnect(self):
         '''Does not try to reconnect connections that are not ready'''
-        with self.mocked_dead_connection() as conn:
-            conn.ready_to_reconnect.return_value = False
-            self.client.check_connections()
-            self.assertFalse(conn.connect.called)
+        conn = self.connections[0]
+        conn.close()
+        conn.ready_to_reconnect.return_value = False
+        self.client.check_connections()
+        self.assertFalse(conn.connect.called)
 
     def test_ready_to_reconnect(self):
         '''Tries to reconnect when ready'''
-        with self.mocked_dead_connection() as conn:
-            conn.ready_to_reconnect.return_value = True
-            self.client.check_connections()
-            conn.connect.assert_called_with()
+        conn = self.connections[0]
+        conn.close()
+        conn.ready_to_reconnect.return_value = True
+        self.client.check_connections()
+        self.assertTrue(conn.connect.called)
 
     def test_set_blocking(self):
         '''Sets blocking to 0 when reconnecting'''
-        with self.mocked_dead_connection() as conn:
-            conn.ready_to_reconnect.return_value = True
-            conn.connect.return_value = True
-            self.client.check_connections()
-            conn.setblocking.assert_called_with(0)
+        conn = self.connections[0]
+        conn.close()
+        conn.ready_to_reconnect.return_value = True
+        conn.connect.return_value = True
+        self.client.check_connections()
+        conn.setblocking.assert_called_with(0)
 
     def test_calls_reconnected(self):
         '''Sets blocking to 0 when reconnecting'''
-        with self.mocked_dead_connection() as conn:
-            conn.ready_to_reconnect.return_value = True
-            conn.connect.return_value = True
-            with mock.patch.object(self.client, 'reconnected'):
-                self.client.check_connections()
-                self.client.reconnected.assert_called_with(conn)
-
-
-if __name__ == '__main__':
-    unittest.main()
+        conn = self.connections[0]
+        conn.close()
+        conn.ready_to_reconnect.return_value = True
+        conn.connect.return_value = True
+        with mock.patch.object(self.client, 'reconnected'):
+            self.client.check_connections()
+            self.client.reconnected.assert_called_with(conn)
